@@ -797,10 +797,21 @@ class CAT(object):
                 for id, text in data:
                     try:
                         if max_chars_in_memory > 0:
-                            while out_dict['current_size_in_chars'] > max_chars_in_memory:
+                            cnt = 0
+                            while (out_dict['current_size_in_chars'] + len(text))) > max_chars_in_memory:
+                                cnt += 1
                                 self.log.debug("Process: {}, waiting because current chars in memory: {}".format(pid, out_dict['current_size_in_chars']))
                                 # Wait until size in memory is low
-                                sleep(1)
+                                sleep(3)
+                                if cnt % 200 == 0:
+                                    self.log.info("Process: {}, waiting because current chars in memory: {}, cnt: {}".format(
+                                        pid, out_dict['current_size_in_chars'], cnt))
+                                if cnt % 2000 == 0:
+                                    self.log.info("Breaking out: {} - a stuck process because current chars in memory: {}, cnt: {}".format(
+                                        pid, out_dict['current_size_in_chars'], cnt))
+                                    # Reset, probably something went wrong somewhere
+                                    out_dict['current_size_in_chars'] = 0
+                                    break
                         out_dict['current_size_in_chars'] = out_dict['current_size_in_chars'] + len(text)
                         self.log.debug("Process: {}, current chars in memory (before) is: {}".format(pid, out_dict['current_size_in_chars']))
                         # Annotate document
@@ -813,5 +824,6 @@ class CAT(object):
                     except Exception as e:
                         self.log.warning("Exception in _mp_cons")
                         self.log.warning(e, stack_info=True)
+                        out_dict['current_size_in_chars'] = out_dict['current_size_in_chars'] - len(text)
 
             sleep(1)
